package ru.dbotthepony.kommons.guava

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableMap
import com.google.common.collect.ImmutableMultimap
import com.google.common.collect.ImmutableSet
import java.util.function.Consumer

inline fun <T : Any> immutableList(size: Int, initializer: (index: Int) -> T): ImmutableList<T> {
	require(size >= 0) { "Invalid list size $size" }

	return when (size) {
		0 -> ImmutableList.of()
		1 -> ImmutableList.of(initializer(0))
		else -> ImmutableList.Builder<T>().let {
			for (i in 0 until size) {
				it.add(initializer(i))
			}

			it.build()
		}
	}
}

inline fun <K : Any, V : Any> immutableMap(initializer: ImmutableMap.Builder<K, V>.() -> Unit): ImmutableMap<K, V> {
	val builder = ImmutableMap.Builder<K, V>()
	initializer.invoke(builder)
	return builder.build()
}

inline fun <K : Any, V : Any> immutableMultimap(initializer: ImmutableMultimap.Builder<K, V>.() -> Unit): ImmutableMultimap<K, V> {
	val builder = ImmutableMultimap.Builder<K, V>()
	initializer.invoke(builder)
	return builder.build()
}

inline fun <V : Any> immutableSet(initializer: Consumer<V>.() -> Unit): ImmutableSet<V> {
	val builder = ImmutableSet.Builder<V>()
	initializer.invoke(builder::add)
	return builder.build()
}

inline fun <V : Any> immutableList(initializer: Consumer<V>.() -> Unit): ImmutableList<V> {
	val builder = ImmutableList.Builder<V>()
	initializer.invoke(builder::add)
	return builder.build()
}

fun <V : Any> immutableList(a: V, vararg values: V): ImmutableList<V> {
	val builder = ImmutableList.Builder<V>()
	builder.add(a)
	builder.addAll(values.iterator())
	return builder.build()
}
